@@ -101,6 +101,11 @@ public:
 
         return pos;
     }
+
+#ifdef move32
+    // One of the system headers right now is sometimes defining a conflicting macro we don't use
+#undef move32
+#endif
     virtual int32_t move32(int32_t delta, CharacterIterator::EOrigin origin) override {
         switch(origin) {
         case kStart:
