@@ -231,7 +231,8 @@ public:
     CompactDecimalFormatTest() {
     }
 
-    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par=0) override;
+    void runIndexedTest(int32_t index, UBool exec, const char*& name, char* par = nullptr) override;
+
 private:
     void TestEnglishShort();
     void TestSerbianShort();
