@@ -1,7 +1,7 @@
 /*
  *
  * © 2016 and later: Unicode, Inc. and others.
- * License & terms of use: http://www.unicode.org/copyright.html#License
+ * License & terms of use: http://www.unicode.org/copyright.html
  *
  * (C) Copyright IBM Corp. 1998-2013 - All Rights Reserved
  *
@@ -10,7 +10,7 @@
 #ifndef __XMLREADER_H
 #define __XMLREADER_H
 
-#include "LETypes.h"
+#include "layout/LETypes.h"
 #include "letest.h"
 
 typedef void (*TestCaseCallback) (const char *testID,
